! vim:fdm=marker:

!-----------------------------------------------------------------------
!> This module contains routines to compute correlation function and 
!> writes them to files, e.g. the one- and two particle densities and
!> momentum distributions
!>
!> \authors Kaspar Sakmann (2007), Axel Lode (2011)
!> \since 2007
MODULE modulecorrelationfunctions

IMPLICIT NONE
CONTAINS
!> wrapper routine which manages output of correlations.
!> compute the first and second order correlation functions in 1D calculations
!> and
!> the first and second order correlation functions in 2D calculations 
!> if desired.
!> \authors Axel Lode, Kaspar Sakmann 
!> \since (2011)
subroutine write_correlations(Dims,printCorrelationFuncs,MOMSPACE2D,REALSPACE2D,&
                        x1const,x1slice,y1const,y1slice,&
                        x2const,x2slice,y2const,y2slice,&
                        AbsTime,Psi,rho_jk,rho_ijkl,& 
                        Pnot,xstart,xstop)
USE moduleinputvariables,ONLY:MOrb,NDX,NDY,NDZ

 IMPLICIT NONE
 
 INTEGER,intent(in) :: Dims,printCorrelationFuncs
 LOGICAL,intent(in) :: MOMSPACE2D, REALSPACE2D
 LOGICAL,intent(in) ::  x1const, x2const, y1const, y2const
 LOGICAL,intent(in) ::  Pnot 
 REAL*8,intent(in)  ::  xstart,xstop
 REAL*8,intent(in)  ::  x1slice, x2slice, y1slice, y2slice
 REAL*8,intent(in)  ::  AbsTime
 COMPLEX*16, DIMENSION(MORB*MORB),intent(in) :: rho_jk
 COMPLEX*16, DIMENSION(MORB*MORB*MORB*MORB),intent(in) :: rho_ijkl
 COMPLEX*16, DIMENSION(NDX*NDY*NDZ,MORB),intent(in) :: Psi

 COMPLEX*16, DIMENSION(NDX*NDY*NDZ,MORB) :: FTPsi
 INTEGER :: dilation 

 dilation=1

 IF(Dims==1) THEN
   if (printCorrelationFuncs==1) then

     CALL get_correlations(AbsTime,Psi,1,1,rho_jk,rho_ijkl) 

     CALL pedestrian_FT(Psi,FTPsi,dilation=1)
     CALL get_dilation(dilation,FTPsi,rho_jk)
     CALL pedestrian_FT(Psi,FTPsi,dilation)
     CALL get_oneBodyRDMDiagonal(AbsTime,FTPsi,rho_jk,2,dilation)
     CALL get_correlations(AbsTime,FTPsi,2,dilation,rho_jk,rho_ijkl) 
     if (Pnot.eqv..TRUE.) then
        CALL  get_densityNonescape(xstart,xstop,rho_jk,AbsTime,Psi)
     endif 
  end if
 ELSE IF (Dims == 2) THEN
   if (MOMSPACE2D.eqv..TRUE.) then
     call FT_SORTED(Psi,FTPSI)
     call get_corr_slice(Abstime,FTPSI,2,rho_jk,rho_ijkl,&
     x1const,x1slice,y1const,y1slice,x2const,x2slice,y2const,y2slice)
   endif
   if (REALSPACE2D.eqv..TRUE.) then
      call get_corr_slice(Abstime,PSI,1,rho_jk,rho_ijkl,&
      x1const,x1slice,y1const,y1slice,x2const,x2slice,y2const,y2slice)
   endif
 END IF
end subroutine write_Correlations


!> This routine integrates the density over one or more cartesian coordinates 
!> and writes the output to a file. Useful for visualizing 2D and 3D densities
!> Which coordinates are integrated out is controlled by the integer writemode=0,...,7
!> the bit representation of writemode indicates which of x,y,z are integrated out, 
!> e.g. writemode=2 is interpreted as 010 and the y-coordinate is integrated out.
!>
!> \authors Kaspar Sakmann
!> \since (2012)
!> 
subroutine get_integratedDensity(rho_jk,AbsTime,PSI,writemode)

USE moduleinputoutput,ONLY: get_doubleAsString
USE moduleinputvariables,ONLY:NDX,NDY,NDZ,MOrb
USE moduleparameters,ONLY: PI
USE moduleallocatables,ONLY:gridx,gridy,gridz

  IMPLICIT NONE

  COMPLEX*16, DIMENSION(NDX*NDY*NDZ,Morb),INTENT(IN)  :: PSI
  complex*16,DIMENSION(Morb,Morb),intent(in) :: rho_jk
  real*8,intent(in) :: AbsTime
  INTEGER,INTENT(IN):: writemode

  complex*16, DIMENSION(NDX*NDY*NDZ)  :: DENSITY
  complex*16, DIMENSION(NDX,NDY,NDZ)  :: int_density
  character(len=10)  :: timeAsString

  integer :: bit,NOGRIDPOINTS,i,j,k,n
  logical :: integratex,integratey,integratez 

  IF(writemode>7 .OR. writemode<0) THEN
    WRITE(*,*)'stupid value for writemode',writemode
    STOP
  END IF
  
  call get_DoubleAsString(AbsTime,timeAsString) 

  int_density = (0.d0,0.d0)
  integratex=.FALSE.
  integratey=.FALSE.
  integratez=.FALSE.

  bit=writemode
  IF(MOD(bit,2)==1) integratez=.TRUE.
  bit=bit/2
  IF(MOD(bit,2)==1) integratey=.TRUE.
  bit=bit/2
  IF(MOD(bit,2)==1) integratex=.TRUE.

  NOGRIDPOINTS=NDX*NDY*NDZ
! get the density from the one-body-matrix elements 
  DENSITY   = (0.d0,0.d0)
  do n=1,NOGRIDPOINTS
     do j = 1,Morb
        do k=1,MOrb
           DENSITY(n) = DENSITY(n) + rho_jk(j,k) * DCONJG(PSI(n,j))*PSI(n,k)
        end do
     end do
  end do

  int_density=(0.d0,0.d0)

  IF (integratez) THEN

    DO i=1,NDX
      DO j=1,NDY
        DO k=1,NDZ
          int_density(i,j,1)=int_density(i,j,1) + DENSITY(i+NDX*(j-1)+NDX*NDY*(k-1)) 
        END DO
      END DO
    END DO

  END IF


  IF (integratey) THEN

    DO i=1,NDX
      DO k=1,NDZ
        DO j=1,NDY
          int_density(i,1,k)=int_density(i,1,k) + DENSITY(i+NDX*(j-1)+NDX*NDY*(k-1)) 
        END DO
      END DO
    END DO

  END IF


  IF (integratex) THEN

    DO k=1,NDZ
      DO j=1,NDY
        DO i=1,NDX
          int_density(1,j,k)=int_density(1,j,k) + DENSITY(i+NDX*(j-1)+NDX*NDY*(k-1)) 
        END DO
      END DO
    END DO

  END IF


 

 
 
6667  format(5(F16.9,a1))



  IF (writemode==0) THEN

    OPEN(unit=12,file=timeAsString//'xyzdensity.dat',form='formatted')

    do k=1,NDZ
      do j=1,NDY
        do i=1,NDX
          write(12,6667) gridx(i)," ",gridy(j)," ",gridz(k)," ",DBLE(int_density(i,j,k))," ",AbsTime

          if(MOD(i,NDX)==0) write(12,*)" " 

        end do
      end do
    end do
                             
    CLOSE(12)

  ELSE IF (writemode==1) THEN

    OPEN(unit=12,file=timeAsString//'xydensity.dat',form='formatted')

    do j=1,NDY
      do i=1,NDX

        write(12,6667) gridx(i)," ",gridy(j)," ",gridz(1)," ",DBLE(int_density(i,j,1))," ",AbsTime

        if(MOD(i,NDX)==0) write(12,*)" " 

      end do
    end do
                             
    CLOSE(12)



  ELSE IF (writemode==2) THEN

    OPEN(unit=12,file=timeAsString//'xzdensity.dat',form='formatted')

    do k=1,NDZ
      do i=1,NDX

        write(12,6667) gridx(i)," ",gridy(1)," ",gridz(k)," ",DBLE(int_density(i,1,k))," ",AbsTime

        if(MOD(i,NDX)==0) write(12,*)" " 

      end do
    end do
                             
    CLOSE(12)


  ELSE IF (writemode==3) THEN 

    OPEN(unit=12,file=timeAsString//'xdensity.dat',form='formatted')

    do i=1,NDX

      write(12,6667) gridx(i)," ",gridy(1)," ",gridz(1)," ",DBLE(int_density(i,1,1))," ",AbsTime

      if(MOD(i,NDX)==0) write(12,*)" " 

    end do
                             
    CLOSE(12)

  ELSE IF (writemode==4) THEN 

    OPEN(unit=12,file=timeAsString//'yzdensity.dat',form='formatted')

    do k=1,NDZ
      do j=1,NDY

        write(12,6667) gridx(1)," ",gridy(j)," ",gridz(k)," ",DBLE(int_density(1,j,k))," ",AbsTime

        if(MOD(j,NDY)==0) write(12,*)" " 

      end do
    end do
                             
    CLOSE(12)

  ELSE IF (writemode==5) THEN 

    OPEN(unit=12,file=timeAsString//'ydensity.dat',form='formatted')

    do j=1,NDY

        write(12,6667) gridx(1)," ",gridy(j)," ",gridz(1)," ",DBLE(int_density(1,j,1))," ",AbsTime

        if(MOD(j,NDY)==0) write(12,*)" " 

    end do
                             
    CLOSE(12)

  ELSE IF (writemode==6) THEN 

    OPEN(unit=12,file=timeAsString//'zdensity.dat',form='formatted')

    do k=1,NDZ

        write(12,6667) gridx(1)," ",gridy(1)," ",gridz(k)," ",DBLE(int_density(1,1,k))," ",AbsTime

        if(MOD(k,NDZ)==0) write(12,*)" " 

    end do
                             
    CLOSE(12)

  ELSE IF (writemode==7) THEN 
 
    OPEN(unit=12,file=timeAsString//'density.dat',form='formatted')

        write(12,6667) gridx(1)," ",gridy(1)," ",gridz(1)," ",DBLE(int_density(1,1,1))," ",AbsTime
                             
    CLOSE(12)
    
  END IF



end subroutine get_integratedDensity


!> This routine computes the nonescape probability or any desired integral of the 
!> diagonal of the reduced one-body density matrix
!> \authors Axel Lode
!> \since (2011)
!> \todo generalize to more than 1D

subroutine get_densityNonescape(xstart,xstop,rho_jk,time,PSI)

USE moduleinputvariables,ONLY:NDX,NDY,NDZ,MOrb
USE moduleparameters,ONLY: PI
USE moduleallocatables,ONLY:gridx,gridy,gridz

  IMPLICIT NONE

  COMPLEX*16, DIMENSION(NDX*NDY*NDZ,Morb)  :: PSI
  complex*16,DIMENSION(Morb,Morb),intent(in) :: rho_jk
  real*8, intent(in) :: xstart,xstop
  real*8 :: nonescape,time,w
  
  complex*16, DIMENSION(NDX*NDY*NDZ)  :: DENSITY    

  integer :: i,j,k,l,m,n,NOGRIDPOINTS


  NOGRIDPOINTS=NDX*NDY*NDZ
! get the density from the one-body-matrix elements 
  DENSITY   = (0.d0,0.d0)
  do n=1,NOGRIDPOINTS
     do j = 1,Morb  
        do k=1,MOrb
           DENSITY(n) = DENSITY(n) + rho_jk(j,k) * DCONJG(PSI(n,j))*PSI(n,k)
        end do
     end do
  end do

! integrate from DENSITY from xi to xf
  nonescape=0.d0
  do n=1,NOGRIDPOINTS
     if ((gridx(n).le.xstop).and.(gridx(n).ge.xstart)) then
        nonescape=nonescape +dble(DENSITY(n))
     endif
  end do
! open/write/close output
  open(unit=542,file='nonescape',form='formatted',ACCESS='APPEND')

  write(542,6545) time,' ',nonescape 

6545  format(2(F21.16,a1))

  close(542)
end subroutine get_densityNonescape




!-----------------------------------------------------------------------
!> This routine computes the maximum of either the one-particle density 
!> or the momentum  distribution depending on whether th array of orbitals 
!> Psi or its Fourier transform FTPsi is given as the first argument. 
!>
!> \authors Kaspar Sakmann (2007)
!> \since 2007
!> \todo make all use associated variables arguments of the routine, variable
!> documentation
subroutine get_maximum(Psi,rho_jk,maximum)
USE moduleinputvariables,ONLY:NDX,NDY,NDZ,MOrb
!---{{{

  implicit none
  integer  :: noGridPoints
  complex*16,DIMENSION(NDX*NDY*NDZ,MOrb),intent(in) :: Psi 
  complex*16,DIMENSION(MOrb,MOrb),intent(in) :: rho_jk
  complex*16,intent(out)  :: maximum
  integer                 :: m,Jorb,Korb
  complex*16              :: G1
  noGridPoints=NDX*NDY*NDZ


  maximum = (0.d0,0.d0)
  do m=1,noGridPoints
 
 !compute G1
     G1 = (0.d0,0.d0)
     do Korb = 1,MOrb  
        do Jorb=1,MOrb

           G1 = G1 + rho_jk(Jorb,Korb) * DCONJG(Psi(m,Jorb))*Psi(m,Korb)

        end do
     end do

     if (REAL(CDABS(G1)) > REAL(CDABS(maximum))) maximum = G1
 
  end do
!---}}}
end subroutine get_maximum

!----------------------------------------------------------------------------
!> This routine computes the diagonal of the first order correlation function
!> G1(x,x) if Psi is provided or G1(k,k) if FT(Psi) is provided and
!> writes it to a file. 
!>
!> time          : current time 
!> Psi           : orbital array 
!> rho_jk        : the 1-body density matrix where rho_jk(j,k)=rho_jk 
!> mode          : mode=1: G1(x,x) 
!>                 mode=2: G1(k,k) is computed
!> dilation      : the integer dilation gives the factor by which the momentum intervall should
!>                 be shrunk or equivalently the factor by which the sampling rate is increased.
!>                 The momentum distribution at the outer ends is padded with zeros then. 
!>                 Normally dilation is computed by get_dilation. If unsure put
!>                 dilation = 1, but the momentum grid will then usually have a bad
!>                 resolution. 
!>
!>\authors Kaspar Sakmann (2007)
!> \since 2007
!>\todo reduce the use associations as much as possible, variable
!> documentation
subroutine get_oneBodyRDMDiagonal(time,Psi,rho_jk,mode,dilation) 
USE modulederivesystemvariables, ONLY: weight
USE moduleinputvariables
USE moduleinputoutput,ONLY: get_doubleAsString
USE moduleparameters,ONLY: PI
USE moduleallocatables,ONLY:gridx,gridy,gridz
USE modulegrid
!---{{{

  implicit none
  integer            :: noGridPoints
  integer,intent(in) :: mode,dilation
  real*8             :: time,w,dk,mom_X(NDX),mom_Y(NDY),mom_Z(NDZ)
  complex*16,DIMENSION(NDX*NDY*NDZ,MOrb),intent(in) :: Psi 
  complex*16,DIMENSION(MOrb,MOrb),intent(in) :: rho_jk
  character(len=2)   :: aString
  character(len=200) :: fullName
  character(len=10)  :: timeAsString
  character(len=10)  :: MOrbAsString
  character(len=1)   :: NparAsString1
  character(len=2)   :: NparAsString2
  character(len=3)   :: NparAsString3
  character(len=4)   :: NparAsString4
  character(len=5)   :: NparAsString5
  character(len=6)   :: NparAsString6
  character(len=7)   :: NparAsString7
  integer            :: n,Jorb,Korb,i_n,j_n,k_n
  complex*16         :: G1n_n
  complex*16         :: maximum
  real*8, parameter  :: TOL=0.00001d0 
  real*8             :: threshold
  real*8             :: outG1n_n


  noGridPoints=NDX*NDY*NDZ


  if((NDY>1).or. (NDZ>1)) then
!     write(*,*)"test of 2D and 3D still necessary"
!     STOP
  end if


  if(mode==1) then
     w = weight
  else if (mode==2) then
     dk     = (2.d0*PI)**Dims/ (weight**2 * noGridPoints)/dilation
     w      = DSQRT( dk )

     CALL get_momentumGrid(xi,xf,NDX,mom_X) !returns momentum grid in strictly ascending order
     CALL get_momentumGrid(yi,yf,NDY,mom_Y)
     CALL get_momentumGrid(zi,zf,NDZ,mom_Z)

     mom_X=mom_X/dilation
     mom_Y=mom_Y/dilation
     mom_Z=mom_Z/dilation

  else 
        write(*,*) "wrong mode"
  end if
  
  write (MOrbAsString, '(I1)') MOrb
  call get_doubleAsString(time,timeAsString) 

  if (mode==1) then
     aString = 'x-'
  else if (mode==2) then 
     aString = 'k-'
  else 
     write(*,*)"ERROR in get_correlations"
     STOP
  endif   
  if ((NPar>0) .and. (NPar.lt.10)) then 
    write (NParAsString1, '(I1)') NPar
    fullName=timeasString//'N'//trim(NParAsString1)//'M'//trim(MOrbAsString)//aString//'density.dat'
  else  if ((NPar.ge.10) .and. (NPar.lt.100)) then 
    write (NParAsString2, '(I2)') NPar
    fullName=timeasString//'N'//trim(NParAsString2)//'M'//trim(MOrbAsString)//aString//'density.dat'
  else if ((NPar.ge.100) .and. (NPar.lt.1000)) then 
    write (NParAsString3, '(I3)') NPar
    fullName=timeasString//'N'//trim(NParAsString3)//'M'//trim(MOrbAsString)//aString//'density.dat'
  else if ((NPar.ge.1000) .and. (NPar.lt.10000)) then 
    write (NParAsString4, '(I4)') NPar
    fullName=timeasString//'N'//trim(NParAsString4)//'M'//trim(MOrbAsString)//aString//'density.dat'
  else if ((NPar.ge.10000) .and. (NPar.lt.100000)) then 
    write (NParAsString5, '(I5)') NPar
    fullName=timeasString//'N'//trim(NParAsString5)//'M'//trim(MOrbAsString)//aString//'density.dat'
  else if ((NPar.ge.100000) .and. (NPar.lt.1000000)) then 
    write (NParAsString6, '(I6)') NPar
    fullName=timeasString//'N'//trim(NParAsString6)//'M'//trim(MOrbAsString)//aString//'density.dat'
  else if ((NPar.ge.1000000) .and. (NPar.lt.10000000)) then 
    write (NParAsString7, '(I7)') NPar
    fullName=timeasString//'N'//trim(NParAsString7)//'M'//trim(MOrbAsString)//aString//'density.dat'
  else 
    NParAsString4='XXXX'
    fullName=timeasString//'N'//trim(NParAsString4)//'M'//trim(MOrbAsString)//aString//'density.dat'
  end if
 ! needs to be edited for other dvrs with nonconstant weights! 
  call get_maximum(Psi,rho_jk,maximum)
  threshold = 0.01d0 * DBLE(maximum)/w**2 ! set the minimal density to be plotted
  
  open(unit=12,file=trim(fullName),form='formatted')
  do n=1,noGridPoints

!compute G1n_n
     G1n_n = (0.d0,0.d0)
     do Jorb = 1,MOrb  
        do Korb=1,MOrb

           G1n_n = G1n_n + rho_jk(Jorb,Korb) * DCONJG(Psi(n,Jorb))*Psi(n,Korb)

        end do
     end do
     G1n_n=G1n_n*1.d0/w**2 
     outG1n_n=DBLE(G1n_n)

     call get_ijk_from_m(n,NDX,NDY,i_n,j_n,k_n)

     if (mode==1) then

         write(12,6667) gridx(i_n)," ",gridy(j_n)," ",gridz(k_n)," ",outG1n_n," ",time

     else if (mode==2) then

         write(12,6667) mom_X(i_n)," ",mom_Y(j_n)," ",mom_Z(k_n)," ", outG1n_n," ",time

     end if

     if(MOD(n,NDX)==0) write(12,*)" " 

  end do
                             
  close(12)
 
 
6667  format(3(F8.3,a1),2(F16.9,a1))
!---}}}
end subroutine get_oneBodyRDMDiagonal

!--------------------------------------------------------------------------------------------
!> This routine computes the correlation functions 
!> G1(r,r), G1(r,r'), G1(r',r') and G2(r,r',r',r), where r=(x,y,z), 
!> and writes them to a file. The first six columns contain
!> x,y,z,x',y',z'. Currently only 1D has been tested.
!>
!> time          : current time 
!> Psi           : orbital array 
!> mode          : mode=1: x-space (input Psi=Psi) 
!>                 mode=2: k-space (input Psi=FT[Psi])
!> dilation      : the integer dilation gives the factor by which the momentum intervall should
!>                 be shrunk or equivalently the factor by which the sampling rate is increased.
!>                 The momentum distribution at the outer ends is padded with zeros then. 
!>                 Normally dilation is computed by get_dilation. If unsure put
!>                 dilation = 1, but the momentum grid will then usually have a bad
!>                 resolution.
!> rho_jk        : the 1-body density matrix where rho_jk(j,k)   = rho_jk 
!> rho_ijkl      : the 2-body density matrix where rho_ijkl(j,k) = rho_ijkl 
!>
!>\authors Kaspar Sakmann (2007)
!> \since 2007
!>\todo: reduce use associations, variable
!> documentation
subroutine get_correlations(time,Psi,mode,dilation,rho_jk,rho_ijkl) 
USE moduleinputoutput,ONLY: get_doubleAsString
USE modulegrid
USE moduleinputvariables
USE moduleparameters,ONLY: PI
USE moduleallocatables,ONLY:gridx,gridy,gridz
!  {{{

  implicit none
  integer            :: noGridPoints
  integer,intent(in) :: mode,dilation
  real*8,intent(in)  :: time
  real*8     :: w,dk,mom_X(NDX),mom_Y(NDY),mom_Z(NDZ)
  complex*16,DIMENSION(NDX*NDY*NDZ,MOrb),intent(in) :: Psi 
  complex*16,intent(in) :: rho_ijkl(MOrb,MOrb,MOrb,MOrb)
  complex*16,intent(in) :: rho_jk(MOrb,MOrb)
  character(len=2)   :: aString
  character(len=200) :: fullName
  character(len=10)  :: timeAsString
  character(len=10)  :: MOrbAsString
  character(len=1)   :: NparAsString1
  character(len=2)   :: NparAsString2
  character(len=3)   :: NparAsString3
  character(len=4)   :: NparAsString4
  integer            :: m,n,Iorb,Jorb,Korb,Lorb,i_m,j_m,k_m,i_n,j_n,k_n
  complex*16         :: G1m_m,G1m_n,G1n_n,G2m_n

  complex*16         :: maximum
  real*8,parameter   :: TOL=0.00001d0 
  real*8     :: threshold
  real*8     :: outG1m_m,outReG1m_n,outImG1m_n,outG1n_n,outG2m_n

  noGridPoints=NDX*NDY*NDZ

  if((NDY>1).or. (NDZ>1)) then
     write(*,*)"only 1d implemented so far"
     STOP
  end if


  if(mode==1) then
     w=DSQRT((xf-xi)/NDX)
  else if (mode==2) then
     dk     = 2.d0*PI/(xf-xi)/dilation
     w      = DSQRT( dk )

     CALL get_momentumGrid(xi,xf,NDX,mom_X) !returns momentum grid in strictly ascending order
     CALL get_momentumGrid(yi,yf,NDY,mom_Y)
     CALL get_momentumGrid(zi,zf,NDZ,mom_Z)

     mom_X=mom_X/dilation
     mom_Y=mom_Y/dilation
     mom_Z=mom_Z/dilation

  else 
        write(*,*) "wrong mode"
  end if
  
  write (MOrbAsString, '(I1)') MOrb
  call get_doubleAsString(time,timeAsString) 

  if (mode==1) then
     aString = 'x-'
  else if (mode==2) then 
     aString = 'k-'
  else 
     write(*,*)"ERROR in get_correlations"
     STOP
  endif   

  if ((NPar>0) .and. (NPar.lt.10)) then 
    write (NParAsString1, '(I1)') NPar
    fullName=timeasString//'N'//trim(NParAsString1)//'M'//trim(MOrbAsString)//aString//'correlations.dat'
  else  if ((NPar.ge.10) .and. (NPar.lt.100)) then 
    write (NParAsString2, '(I2)') NPar
    fullName=timeasString//'N'//trim(NParAsString2)//'M'//trim(MOrbAsString)//aString//'correlations.dat'
  else if ((NPar.ge.100) .and. (NPar.lt.1000)) then 
    write (NParAsString3, '(I3)') NPar
    fullName=timeasString//'N'//trim(NParAsString3)//'M'//trim(MOrbAsString)//aString//'correlations.dat'
  else if ((NPar.ge.1000) .and. (NPar.lt.10000)) then 
    write (NParAsString4, '(I4)') NPar
    fullName=timeasString//'N'//trim(NParAsString4)//'M'//trim(MOrbAsString)//aString//'correlations.dat'
  else 
    NParAsString4='XXXX'
    fullName=timeasString//'N'//trim(NParAsString4)//'M'//trim(MOrbAsString)//aString//'correlations.dat'
  end if
 ! needs to be edited for other dvrs with nonconstant weights! 
  call get_maximum(Psi,rho_jk,maximum)
  threshold = 0.01d0 * DBLE(maximum)/w**2 ! set the minimal density to be plotted
!  threshold = 0.d0
  
  open(unit=12,file=trim(fullName),form='formatted')
  do n=1,noGridPoints

!compute G1n_n
     G1n_n = (0.d0,0.d0)
     do Jorb = 1,MOrb  
        do Korb=1,MOrb

           G1n_n = G1n_n + rho_jk(Jorb,Korb) * DCONJG(Psi(n,Jorb))*Psi(n,Korb)

        end do
     end do
     G1n_n=G1n_n*1.d0/w**2 
     outG1n_n=DBLE(G1n_n)
!---------------
     do m=1,noGridPoints         

!compute G1m_m
        G1m_m = (0.d0,0.d0) 
        do Jorb = 1,MOrb  
           do Korb=1,MOrb

              G1m_m = G1m_m + rho_jk(Jorb,Korb) * DCONJG(Psi(m,Jorb))*Psi(m,Korb)

           end do
        end do
        G1m_m=G1m_m*1.d0/w**2 
        outG1m_m=DBLE(G1m_m)   
!----------------
        
!compute G1m_n
        G1m_n = (0.d0,0.d0) 
        do Jorb = 1,MOrb  
           do Korb=1,MOrb

              G1m_n = G1m_n + rho_jk(Jorb,Korb) * DCONJG(Psi(m,Jorb))*Psi(n,Korb)

           end do
        end do
        G1m_n=G1m_n*1.d0/(w**2) 
        outReG1m_n=DBLE(G1m_n)   
        outImG1m_n=DIMAG(G1m_n)
!----------------


!compute G2m_n 
        G2m_n=(0.d0,0.d0)
        do Lorb=1,MOrb
           do Korb=1,MOrb
              do Jorb=1,MOrb
                 do Iorb=1,MOrb
!                      
                    G2m_n = G2m_n + rho_ijkl(Iorb,Jorb,Korb,Lorb)* &
                              DCONJG(Psi(m,Iorb))*DCONJG(Psi(n,Jorb))*Psi(m,Korb)*Psi(n,Lorb)
  
                 end do
              end do
           end do
        end do
        G2m_n=G2m_n*1.d0/(w**4) 
        outG2m_n=DBLE(G2m_n)
!----------------

        IF((DABS(outG1m_m).LT. MAX(TOL,threshold))     .OR.   &
           (DABS(outG1n_n).LT. MAX(TOL,threshold)) )   THEN 
           outReG1m_n =  -600.d0    
           outImG1m_n =  -600.d0    
           outG2m_n   = -1.d0    
        END IF

        call get_ijk_from_m(m,NDX,NDY,i_m,j_m,k_m)
        call get_ijk_from_m(n,NDX,NDY,i_n,j_n,k_n)

        if (mode==1) then
           write(12,6666) gridx(i_m)," ",gridy(j_m)," ",gridz(k_m)," ",gridx(i_n)," ",gridy(j_n)," ",gridz(k_n)," ", &
                       outG1m_m," ",outReG1m_n," ",outImG1m_n," ",outG1n_n," ",outG2m_n
        else if (mode==2) then

           write(12,6666) mom_X(i_m)," ",mom_Y(j_m)," ",mom_Z(k_m)," ",mom_X(i_n)," ",mom_Y(j_n)," ",mom_Z(k_n)," ", &
                       outG1m_m," ",outReG1m_n," ",outImG1m_n," ",outG1n_n," ",outG2m_n
        end if

        if(MOD(m,NDX)==0) write(12,*)" " 

     end do
                             
  end do         
  close(12)
 
 
6666  format(6(F8.3,a1),6(F16.9,a1))
!---}}}
end subroutine get_correlations


!--------------------------------------------------------------
!> This routine was written to deal with the following problem.
!> If the x-domain has length L, then the k-domain has a grid 
!> spacing dp=2PI/L. It often turns out that this spacing is very 
!> wide. However, if the wave function in x-space is negligibly small
!> at the box boundaries, it can be assumed to be zero outside
!> the box boundaries. Thereby, the length L can be artificially increased
!> and the momentum distribution can be evaluated using a smaller grid.
!> The factor dilation is integer and describes how many times the 
!> k-space interval can be made smaller so that still 0.99% fit into it, 
!> using the same number of grid points.
!> Using get_dilation allows to make nice plots of the momentum distributions.
!> It should only be used to generate nicer output though.
!> If this is unwanted please set dilation = 1 everywhere the variable appears.
!> This is the default.
!>
!>\authors Kaspar Sakmann (2007)
!> \since 2007
!>\todo get rid of use associations, variable
!> documentation
subroutine get_dilation(dilation,Psi,rho_jk)
USE moduleinputvariables,ONLY: NDX,NDY,NDZ,MOrb
!       {{{
  implicit none  
  integer,intent(inout)     :: dilation
  complex*16,DIMENSION(NDX*NDY*NDZ,MOrb),intent(in) :: Psi 
  complex*16,DIMENSION(MOrb,MOrb),intent(in) :: rho_jk

  integer                   :: m,iLower,iUpper
  integer                   :: Jorb,Korb
  complex*16                :: cmax,G1m_m
  real*8                    :: G1max,THRESHOLD,G1(NDX*NDY*NDZ)
  real*8                    :: imin,imax

  if((NDY>1) .or. (NDZ>1)) then
     write(*,*)"only 1d implemented"
     STOP
  end if 

  THRESHOLD =       0.01d0       
  dilation  =       1
  G1max     =       0.d0

  call get_maximum(Psi,rho_jk,cmax)
  G1max = DBLE(cmax)

!---compute G1m_m

     do m=1,NDX*NDY*NDZ 

        G1m_m = (0.d0,0.d0) 
        do Jorb = 1,MOrb  
           do Korb=1,MOrb

              G1m_m = G1m_m + rho_jk(Jorb,Korb) * DCONJG(Psi(m,Jorb))*Psi(m,Korb)

           end do
        end do

        G1(m)=DBLE(G1m_m)   

     end do 

!----------------
!----find boundaries

      do m=1,NDX/2
        if(G1(m)/G1max > THRESHOLD) THEN
          iLower = m  
          iUpper = NDX-m+1
          exit
        end if
        if(G1(NDX-m+1)/G1max > THRESHOLD) THEN
          iLower = m  
          iUpper = NDX-m+1
          exit
        end if  
      end do 

        imax     = 1.d0*(iUpper-(NDX)/2 - 1)
        imin     = 1.d0*(iLower-(NDX)/2 - 1)
        imax     = MAX(DABS(imax),DABS(imin))    
        dilation = MAX(INT(NDX/2.d0 /(2.d0*imax)),1)

!!}}}      
end  subroutine get_dilation

!------------------------------------------------------------------------------
!> This routine computes the Fourier integral FTPsi of the orbital array Psi 
!> and normalizes it to one. If dilation=1 the FFT grid is used, i.e, 
!> the spacing is dp=2pi/L where L is the length of the grid in x-space.
!> Otherwise the integral is evaluated using te spacing 2pi/(dilation*L).
!> Note that this is only OK, if it is clear for some physical reason (e.g. a trap potential)
!> that the wave function is zero outside the intervall of length L.
!>
!> \authors Kaspar Sakmann (2007)
!> \since 2007
!> \todo variable documentation
subroutine pedestrian_FT(Psi,FTPsi,dilation) 
USE moduleinputvariables
USE moduleparameters,ONLY: PI
USE moduleallocatables,ONLY:gridx,gridy,gridz
!---{{{
  implicit none
  integer                  :: Iorb,m,n

  complex*16,DIMENSION(NDX*NDY*NDZ,MOrb),intent(in)  :: Psi 
  complex*16,DIMENSION(NDX*NDY*NDZ,MOrb),intent(out) :: FTPsi
 
  real*8,dimension(NDX*NDY*NDZ) :: kvecs
  real*8                        :: xall,xfirst,xlast,dk,dx,kmin,k,norm
  integer,intent(in)                    :: dilation

!---{{{ check for consistency
  if ((NDY>1) .or. (NDZ>1)) then
     write(*,*) "more than 1-D has not been implemented yet"
     STOP
  end if
!---}}}

  xfirst = xi
  xall   = xf-xi
  dx     = xall/NDX 
  dk     = 2.d0*PI/(xall*dilation) 
  kmin   = -(NDX/2) * dk
  
  FTPsi=(0.d0,0.d0)

  do Iorb=1,MOrb


     do m=1,NDX   

        k = kmin+(m-1)*dk

        do n=1,NDX
           FTPsi(m,Iorb) = FTPsi(m,Iorb) + dx * Psi(n,Iorb)/DSQRT(dx*2.d0*PI) * CDEXP( (0.d0,-1.d0)*k*gridx(n) )
        end do 

     end do 

     call normvxz(FTPsi(:,Iorb),norm,NDX)
     call xvixdzo(norm,FTPsi(:,Iorb),NDX)

  end do


!---}}}
end subroutine pedestrian_FT

!--------------------------------------------------------------------------------------------
!> computes cuts of the correlation functions 
!> G1(r,r), G1(r,r'), G1(r',r') and G2(r,r',r',r), where r=(x,y), 
!> and writes them to a file.  
!> 
!> the user has to choose two of the coordinates to stay constant, to obtain a 3D plot.
!> x1,y1,x2,y2. 
!>
!>
!> time          : current time 
!> PSI           : orbital array 
!> mode          : mode=1: x-space (input PSI=PSI) 
!>                 mode=2: k-space (input PSI=FT[PSI])
!> rho_jk        : the 1-body density matrix where rho_jk(j,k)   = rho_jk 
!> rho_ijkl      : the 2-body density matrix where rho_ijkl(j,k) = rho_ijkl 
!>
!> \authors Axel Lode  (2011), Kaspar Sakmann (2007)
!> \todo variable documentation, variable cleaning
subroutine get_corr_slice(time,PSI,mode,rho_jk,rho_ijkl,&
                       x1const,x1slice,y1const,y1slice,x2const,x2slice,y2const,y2slice) 
!  {{{
USE moduleinputoutput,ONLY: get_doubleAsString
USE modulegrid
use modulederivesystemvariables, ONLY: get_weight
USE moduleinputvariables,ONLY: NDX,NDY,NDZ,morb,dims,NPAR,xi,xf,yi,yf,zi,zf
USE moduleparameters,ONLY: PI
USE moduleallocatables,ONLY:gridx,gridy,gridz
  implicit none
  integer  :: NOGRIDPOINTS
  integer,intent(in) :: mode
  real*8,intent(in)  :: time
  real*8,intent(in)  :: x1slice,x2slice,y1slice,y2slice
  logical,intent(in)  :: x1const,y1const,x2const,y2const
  real*8     :: w,dk,mom_X(NDX),mom_Y(NDY),mom_Z(NDZ)
  COMPLEX*16, DIMENSION(NDX*NDY*NDZ,Morb)  :: PSI
  complex*16,intent(in) :: rho_ijkl(Morb,Morb,Morb,Morb)
  complex*16,intent(in) :: rho_jk(Morb,Morb)
  character(len=2)   :: aString
  character(len=200) :: fullName   
  character(len=10)  :: timeAsString
  character(len=7)  :: slice1AsString
  character(len=7)  :: slice2AsString
  real*8            :: slice1,slice2
  integer           :: slice1pt,slice2pt
  character(len=10)  :: MorbAsString
  character(len=1)   :: NparAsString1
  character(len=2)   :: NparAsString2
  character(len=3)   :: NparAsString3
  character(len=4)   :: NparAsString4
  integer            :: m,n,Iorb,Jorb,Korb,Lorb,i_m,j_m,k_m,i_n,j_n,k_n,k_k
  complex*16         :: G1m_m,G1m_n,G1n_n,G2m_n
  Real*8             :: onebdens(NDX*NDY)

  complex*16         :: maximum
  real*8,parameter   :: TOL=0.00001d0 
  real*8     :: threshold
  real*8     :: outG1m_m,outReG1m_n,outImG1m_n,outG1n_n,outG2m_n


  if (NDY.eq.1) then
     write(*,*) 'The sliced correlations are implemented for >=2D systems only!!'
     return
  endif
  

  NOGRIDPOINTS=NDX*NDY*NDZ
  if(mode.eq.1) then
     CALL get_weight(xi,xf,yi,yf,zi,zf,NDX,NDY,NDZ,Dims,w)
  else if (mode.eq.2) then
     CALL get_momentumGrid(xi,xf,NDX,mom_X) !returns momentum grid in strictly ascending order
     CALL get_momentumGrid(yi,yf,NDY,mom_Y)
     CALL get_momentumGrid(zi,zf,NDZ,mom_Z)
      
     dk     = (mom_Y(2)-mom_y(1))*(mom_X(2)-mom_X(1))
     w      = DSQRT( dk )
  else 
        write(*,*) "wrong mode"
  end if
 

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! some exceptions 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  if (mode.eq.1) then

  if ((x1const.eqv..true.).and.((x1slice.lt.xi).or.(x1slice.gt.xf))) then
     write(*,*) 'The wanted x1 slice of the correlations is outside the grid!!!'
     return 
  endif
  if ((x2const.eqv..true.).and.((x2slice.lt.xi).or.(x2slice.gt.xf))) then
     write(*,*) 'The wanted x1 slice of the correlations is outside the grid!!!'
     return 
  endif
  if ((y1const.eqv..true.).and.((y1slice.lt.yi).or.(y1slice.gt.yf))) then
     write(*,*) 'The wanted x1 slice of the correlations is outside the grid!!!'
     return 
  endif
  if ((y2const.eqv..true.).and.((y2slice.lt.xi).or.(y2slice.gt.xf))) then
     write(*,*) 'The wanted x1 slice of the correlations is outside the grid!!!'
     return 
  endif

  else if (mode.eq.2) then

  if ((x1const.eqv..true.).and.((x1slice.gt.mom_x(NDX)).or.(x1slice.lt.mom_x(1)))) then
     write(*,*) 'The wanted x1 slice of the correlations is outside the grid!!!'
     return 
  endif
  if ((x2const.eqv..true.).and.((x2slice.gt.mom_x(ndx)).or.(x2slice.lt.mom_x(1)))) then
     write(*,*) 'The wanted x1 slice of the correlations is outside the grid!!!'
     return 
  endif
  if ((y1const.eqv..true.).and.((y1slice.gt.mom_y(ndx)).or.(y1slice.lt.mom_y(1)))) then
     write(*,*) 'The wanted x1 slice of the correlations is outside the grid!!!'
     return 
  endif
  if ((y2const.eqv..true.).and.((y2slice.gt.mom_y(ndx)).or.(y2slice.lt.mom_y(1)))) then
     write(*,*) 'The wanted x1 slice of the correlations is outside the grid!!!'
     return 
  endif

  endif 
  m=0


  if (x1const.eqv..true.) then
      m=m+1
      call getsliceasstring(1,x1slice,slice1AsString)
      slice1=x1slice
  endif
  if (y1const.eqv..true.) then
      m=m+1
      if (m.eq.1) then
         call getsliceasstring(2,y1slice,slice1AsString)
         slice1=y1slice
      endif
      if (m.eq.2) then
         call getsliceasstring(2,y1slice,slice2AsString)
         slice2=y1slice
      endif
  endif
  if (x2const.eqv..true.) then 
      m=m+1
      if (m.eq.1) then 
         call getsliceasstring(3,x2slice,slice1AsString)
         slice1=x2slice
      endif
      if (m.eq.2) then 
        call getsliceasstring(3,x2slice,slice2AsString)
        slice2=x2slice
      endif
  endif
  if (y2const.eqv..true.) then 
     m=m+1
     if (m.eq.1) then
        call getsliceasstring(4,y2slice,slice1AsString)
        slice1=y2slice
      endif
     if (m.eq.2) then
        call getsliceasstring(4,y2slice,slice2AsString)
        slice2=y2slice
      endif
  endif

  if (m.ne.2) then
      write(*,*) 'You must keep two of the four coordinates of the correlations constant!!!'
      return
  endif

 
  if (MORB.lt.10) then
     write (MorbAsString, '(I1)') Morb
  else if ((MORB.ge.10).and.(MORB.lt.100)) then
     write (MorbAsString, '(I2)') Morb
  else 
     write(*,*) 'Bigger orbital number than 100 not implemented!'
  endif

  call get_DoubleAsString(time,timeAsString) 

  if (mode.eq.1) then
     aString = 'x-'
  else if (mode.eq.2) then 
     aString = 'k-'
  else 
     write(*,*)"ERROR in get_correlations"
     stop
  endif   
!   assemble filename
  if ((NPar.gt.0) .and. (NPar.lt.10)) then 
    write (NParAsString1, '(I1)') NPar
    fullName=timeasString//'N'//trim(NParAsString1)//'M'//trim(MorbAsString)//aString//&
                          slice1Asstring//'-'//slice2Asstring//'-'//'correlations.dat'
  else  if ((NPar.ge.10) .and. (NPar.lt.100)) then 
    write (NParAsString2, '(I2)') NPar
    fullName=timeasString//'N'//trim(NParAsString2)//'M'//trim(MorbAsString)//aString//&
                          slice1Asstring//'-'//slice2Asstring//'-'//'correlations.dat'
  else if ((NPar.ge.100) .and. (NPar.lt.1000)) then 
    write (NParAsString3, '(I3)') NPar
    fullName=timeasString//'N'//trim(NParAsString3)//'M'//trim(MorbAsString)//aString//&
                          slice1Asstring//'-'//slice2Asstring//'-'//'correlations.dat'
  else if ((NPar.ge.1000) .and. (NPar.lt.10000)) then 
    write (NParAsString4, '(I4)') NPar
    fullName=timeasString//'N'//trim(NParAsString4)//'M'//trim(MorbAsString)//aString//&
                          slice1Asstring//'-'//slice2Asstring//'-'//'correlations.dat'
  else 
    NParAsString4='XXXX'
    fullName=timeasString//'N'//trim(NParAsString4)//'M'//trim(MorbAsString)//aString//&
                          slice1Asstring//'-'//slice2Asstring//'-'//'correlations.dat'
  end if
  
  open(unit=12,file=trim(fullName),form='formatted')
  do n=1,NOGRIDPOINTS
!compute one body density onebdens 
     G1n_n = (0.d0,0.d0)
     do Jorb = 1,Morb  
        do Korb=1,Morb
           G1n_n = G1n_n + rho_jk(Jorb,Korb) * DCONJG(PSI(n,Jorb))*PSI(n,Korb)
        end do
     end do
     G1n_n=G1n_n*1.d0/w**2 
     onebdens(n)=DBLE(G1n_n)
!---------------
   end do

! determine which points are to be plotted    
   do n=1,NOGRIDPOINTS
      call get_ijk_from_m(n,NDX,NDY,i_n,j_n,k_n)

!     x1slice
      if ((mode.eq.1).and.((abs(gridX(i_n)-x1slice).lt.gridX(2)-gridX(1)).and.(x1const.eqv..true.))&
     .or.((mode.eq.2).and.(abs(mom_x(i_n)-x1slice).lt.mom_X(2)-mom_X(1)).and.(x1const.eqv..true.))) then
          call get_ijk_from_m(n+1,NDX,NDY,i_m,j_m,k_m) 
          if ((abs(gridX(i_n)-x1slice)).lt.(abs(gridX(i_m)-x1slice))) then
             slice1pt=i_n
          else
             slice1pt=i_m
          endif
      endif

!     y1slice
      if (((mode.eq.1).and.(abs(gridY(j_n)-y1slice).lt.gridY(2)-gridY(1)).and.(y1const.eqv..true.)).or.&
      ((mode.eq.2).and.(abs(mom_Y(j_n)-y1slice).lt.mom_Y(2)-mom_Y(1)).and.(y1const.eqv..true.))) then
          call get_ijk_from_m(n+1,NDX,NDY,i_m,j_m,k_m) 
          if ((abs(gridY(j_n)-y1slice)).lt.(abs(gridY(j_m)-y1slice))) then
             if (x1const.eqv..false.) slice1pt=j_n
             if (x1const.eqv..true.) then
                 slice2pt=j_n
                 goto 8855
             endif
          else
             if (x1const.eqv..false.) slice1pt=j_m
             if  (x1const.eqv..true.) then 
                slice2pt=j_m
                goto 8855
             endif
          endif
      endif

!     x2slice
      if (((mode.eq.1).and.(abs(gridX(i_n)-x2slice).lt.gridx(2)-gridx(1)).and.(x2const.eqv..true.)).or.&
       ((mode.eq.2).and.(abs(mom_X(i_n)-x2slice).lt.mom_x(2)-mom_x(1)).and.(x2const.eqv..true.))) then
          call get_ijk_from_m(n+1,NDX,NDY,i_m,j_m,k_m) 
          if ((abs(gridx(i_n)-x2slice)).lt.(abs(gridX(i_m)-x2slice))) then
             if ((x1const.eqv..false.).and.(y1const.eqv..false.)) slice1pt=i_n
             if ((x1const.eqv..true.).or.(y1const.eqv..true.)) then
                 slice2pt=i_n
                 goto 8855
             endif
          else
             if ((x1const.eqv..false.).and.(y1const.eqv..false.)) slice1pt=j_m
             if  ((x1const.eqv..true.).or.(y1const.eqv..true.)) then 
                slice2pt=j_m
                goto 8855
             endif
          endif
      endif

!     y2slice
      if (((mode.eq.1).and.(abs(gridY(j_n)-y2slice).lt.gridY(2)-gridy(1)).and.(y2const.eqv..true.)).or.&
      ((mode.eq.2).and.(abs(mom_Y(j_n)-y2slice).lt.mom_Y(2)-mom_Y(1)).and.(y2const.eqv..true.))) then 
          call get_ijk_from_m(n+1,NDX,NDY,i_m,j_m,k_m) 
          if ((abs(gridY(j_n)-y2slice)).lt.(abs(gridY(j_m)-y2slice))) then
             if ((x1const.eqv..true.).or.(y1const.eqv..true.).or.((x2const.eqv..true.))) then
                 slice2pt=j_n
                 goto 8855
             else 
                 write(*,*) 'Something went wrong with the assignment of the slices!!!'
                 stop
             endif
          else
             if ((x1const.eqv..true.).or.(y1const.eqv..true.).or.((x2const.eqv..true.))) then
                slice2pt=j_m
                goto 8855
             else 
                 write(*,*) 'Something went wrong with the assignment of the slices!!!'
                 stop
             endif
          endif
       endif
   end do
 8855 continue


 do n=1,NOGRIDPOINTS
   call get_ijk_from_m(n,NDX,NDY,i_n,j_n,k_n)
!  x1const and y1const
   if (((x1const.eqv..true.).and.(i_n.eq.slice1pt)).and.&
      ((y1const.eqv..true.).and.(j_n.eq.slice2pt))) then
   do m=1,NOGRIDPOINTS
      call get_ijk_from_m(m,NDX,NDY,i_m,j_m,k_m)

!compute G1m_n
        G1m_n = (0.d0,0.d0) 
        do Jorb = 1,Morb  
           do Korb=1,Morb
              G1m_n = G1m_n + rho_jk(Jorb,Korb) * DCONJG(PSI(m,Jorb))*PSI(n,Korb)
           end do
        end do
        G1m_n=G1m_n*1.d0/(w**2) 
        outReG1m_n=DBLE(G1m_n)   
        outImG1m_n=DIMAG(G1m_n)
!----------------


!compute G2m_n 
        G2m_n=(0.d0,0.d0)
        do Lorb=1,Morb
           do Korb=1,Morb
              do Jorb=1,Morb
                 do Iorb=1,Morb
                    G2m_n = G2m_n + rho_ijkl(Iorb,Jorb,Korb,Lorb)* &
                              DCONJG(PSI(m,Iorb))*DCONJG(PSI(n,Jorb))*PSI(m,Korb)*PSI(n,Lorb)
  
                 end do
              end do
           end do
        end do
        G2m_n=G2m_n*1.d0/(w*w*w*w) 
        outG2m_n=DBLE(G2m_n)
!----------------

        if (mode.eq.1) then
           write(12,6666) gridx(i_m),gridy(j_m),gridx(i_n),gridy(j_n), &
                       onebdens(m),onebdens(n),outReG1m_n,outImG1m_n,outG2m_n
        else if (mode.eq.2) then
           write(12,6666) mom_X(i_m),mom_Y(j_m),mom_X(i_n),mom_Y(j_n), &
                       onebdens(m),onebdens(n),outReG1m_n,outImG1m_n,outG2m_n
        end if
     if (mod(m,ndx).eq.0)     write(12,*)"                                                       " 

     end do
! if x1const and x2const or y2const
     else if ((x1const.eqv..true.).and.((x2const.eqv..true.).or.(y2const.eqv..true.))&
              .and.(i_n.eq.slice1pt)) then 
     do m=1,NOGRIDPOINTS

        call get_ijk_from_m(m,NDX,NDY,i_m,j_m,k_m)
! if the point is in the slice
        if (((x2const.eqv..true.).and.(i_m.eq.slice2pt))&
         .or.((y2const.eqv..true.).and.(j_m.eq.slice2pt))) then
!compute G1m_n
        G1m_n = (0.d0,0.d0) 
        do Jorb = 1,Morb  
           do Korb=1,Morb
              G1m_n = G1m_n + rho_jk(Jorb,Korb) * DCONJG(PSI(m,Jorb))*PSI(n,Korb)
           end do
        end do
        G1m_n=G1m_n*1.d0/(w**2) 
        outReG1m_n=DBLE(G1m_n)   
        outImG1m_n=DIMAG(G1m_n)
!----------------


!compute G2m_n 
        G2m_n=(0.d0,0.d0)
        do Lorb=1,Morb
           do Korb=1,Morb
              do Jorb=1,Morb
                 do Iorb=1,Morb
                    G2m_n = G2m_n + rho_ijkl(Iorb,Jorb,Korb,Lorb)* &
                              DCONJG(PSI(m,Iorb))*DCONJG(PSI(n,Jorb))*PSI(m,Korb)*PSI(n,Lorb)
  
                 end do
              end do
           end do
        end do
        G2m_n=G2m_n*1.d0/(w*w*w*w) 
        outG2m_n=DBLE(G2m_n)
!----------------

        if (mode.eq.1) then
           write(12,6666) gridx(i_m),gridy(j_m),gridx(i_n),gridy(j_n), &
                       onebdens(m),onebdens(n),outReG1m_n,outImG1m_n,outG2m_n
        else if (mode.eq.2) then
           write(12,6666) mom_X(i_m),mom_Y(j_m),mom_X(i_n),mom_Y(j_n), &
                       onebdens(m),onebdens(n),outReG1m_n,outImG1m_n,outG2m_n
        end if

        endif ! loops only executed if point is in the slice
     enddo
     write(12,*) "                                                " 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!  if x1const false, y1const true and x2/y2const true
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
     elseif ((x1const.eqv..false.).and.(y1const.eqv..true.).and.(j_n.eq.slice1pt)) then

     do m=1,NOGRIDPOINTS
        call get_ijk_from_m(m,NDX,NDY,i_m,j_m,k_m)
! if the point is in the slice
        if (((x2const.eqv..true.).and.(i_m.eq.slice2pt))&
         .or.((y2const.eqv..true.).and.(j_m.eq.slice2pt))) then
!compute G1m_n
        G1m_n = (0.d0,0.d0) 
        do Jorb = 1,Morb  
           do Korb=1,Morb
              G1m_n = G1m_n + rho_jk(Jorb,Korb) * DCONJG(PSI(m,Jorb))*PSI(n,Korb)
           end do
        end do
        G1m_n=G1m_n*1.d0/(w**2) 
        outReG1m_n=DBLE(G1m_n)   
        outImG1m_n=DIMAG(G1m_n)

!compute G2m_n 
        G2m_n=(0.d0,0.d0)
        do Lorb=1,Morb
           do Korb=1,Morb
              do Jorb=1,Morb
                 do Iorb=1,Morb
                    G2m_n = G2m_n + rho_ijkl(Iorb,Jorb,Korb,Lorb)* &
                              DCONJG(PSI(m,Iorb))*DCONJG(PSI(n,Jorb))*PSI(m,Korb)*PSI(n,Lorb)
  
                 end do
              end do
           end do
        end do
        G2m_n=G2m_n*1.d0/(w*w*w*w) 
        outG2m_n=DBLE(G2m_n)
!----------------

        if (mode.eq.1) then
           write(12,6666) gridx(i_m),gridy(j_m),gridx(i_n),gridy(j_n), &
                       onebdens(m),onebdens(n),outReG1m_n,outImG1m_n,outG2m_n
        else if (mode.eq.2) then
           write(12,6666) mom_X(i_m),mom_Y(j_m),mom_X(i_n),mom_Y(j_n), &
                       onebdens(m),onebdens(n),outReG1m_n,outImG1m_n,outG2m_n
        end if

        endif ! loops only executed if point is in the slice
     enddo
      write(12,*)"                                                 " 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!  if x1const false, y1const false and x2const and y2const true
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
     elseif ((x1const.eqv..false.).and.(y1const.eqv..false.)) then
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
     do m=1,NOGRIDPOINTS
        call get_ijk_from_m(m,NDX,NDY,i_m,j_m,k_m)
! if the point is in the slice
        if (((x2const.eqv..true.).and.(i_m.eq.slice1pt))&
         .and.((y2const.eqv..true.).and.(j_m.eq.slice2pt))) then
!compute G1m_n
        G1m_n = (0.d0,0.d0) 
        do Jorb = 1,Morb  
           do Korb=1,Morb
              G1m_n = G1m_n + rho_jk(Jorb,Korb) * DCONJG(PSI(m,Jorb))*PSI(n,Korb)
           end do
        end do
        G1m_n=G1m_n*1.d0/(w**2) 
        outReG1m_n=DBLE(G1m_n)   
        outImG1m_n=DIMAG(G1m_n)
!----------------


!compute G2m_n 
        G2m_n=(0.d0,0.d0)
        do Lorb=1,Morb
           do Korb=1,Morb
              do Jorb=1,Morb
                 do Iorb=1,Morb
                    G2m_n = G2m_n + rho_ijkl(Iorb,Jorb,Korb,Lorb)* &
                              DCONJG(PSI(m,Iorb))*DCONJG(PSI(n,Jorb))*PSI(m,Korb)*PSI(n,Lorb)
  
                 end do
              end do
           end do
        end do
        G2m_n=G2m_n*1.d0/(w*w*w*w) 
        outG2m_n=DBLE(G2m_n)
!----------------
        if (mode.eq.1) then
           write(12,6666) gridx(i_m),gridy(j_m),gridx(i_n),gridy(j_n), &
                       onebdens(m),onebdens(n),outReG1m_n,outImG1m_n,outG2m_n
        else if (mode.eq.2) then
           write(12,6666) mom_X(i_m),mom_Y(j_m),mom_X(i_n),mom_Y(j_n), &
                       onebdens(m),onebdens(n),outReG1m_n,outImG1m_n,outG2m_n
        end if

        endif ! loops only executed if point is in the slice
     enddo
     write(12,*)"                                                        " 
     endif !loop m only if n is in the slice
  end do         

 6666 FORMAT(9(E20.10))

  close(12)
 
!---}}}
end subroutine get_corr_slice
!>--------------------------------------------------------------------------------------------
!> This subroutine uses the FFTW/ACML routines to Fourier-transform all the Orbitals and
!> returns them in strictly ascending order for analysis purposes.
!> \authors Axel Lode  (2011)
!> \since 2011
!> \todo Probably move this routine to the FFT module
subroutine FT_SORTED(PSI,FTPSI) 
!---{{{
USE moduleinputvariables
USE moduleparameters,ONLY: PI
USE moduleallocatables,ONLY:gridx,gridy,gridz
USE modulefft

  implicit none

  integer                  :: Iorb,m,n,i,j,k,INFO

  complex*16,DIMENSION(NDX*NDY*NDZ,Morb),intent(in)  :: PSI 
  complex*16,DIMENSION(NDX*NDY*NDZ,Morb),intent(out) :: FTPSI
  complex*16,DIMENSION(NDX*NDY*NDZ) :: ORBTMP 
 
  real*8,dimension(NDX*NDY*NDZ) :: kvecs
  real*8   :: xall,xfirst,xlast,dk,dx,kmin,norm
  real*8 :: nrm

   if (NDZ.gt.1) then
     write(*,*) "more than 2-D FFT sorting has not been implemented yet"
     return
   end if
   !##########################################################  
   !##########################################################  
   !     1D -CASE
   if (dims.eq.1) then
   !##########################################################  
   !##########################################################  
   FTPSI=PSI
   do Iorb=1,Morb
         ORBTMP=(FTPSI(:,IORB))
         CALL get_FFT1D(-1,NDX,ORBTMP,INFO)

         if (mod(NDX,2).eq.0) then
           do m=(NDX/2+1),NDX
              FTPSI(m-(NDX/2),IORB)=ORBTMP(m)
              FTPSI(m,IORB)=ORBTMP(m-(NDX/2))
           enddo
         else
           do m=(NDX/2+1),NDX
              FTPSI(m-(NDX/2),IORB)=ORBTMP(m)
              FTPSI(m,IORB)=ORBTMP(m-(NDX/2))
           enddo
         endif

     call normvxz(FTPSI(:,Iorb),nrm,NDX)
     call xvixdzo(nrm,FTPSI(:,Iorb),NDX)

  end do
  
  !##########################################################  
  !##########################################################  
  !     1D -CASE
  endif 
  !##########################################################  
  !##########################################################  

  !##########################################################  
  !##########################################################  
  !     2D -CASE
  if (DIMs.eq.2) then
  !##########################################################  
  !##########################################################  
  FTPSI=PSI
     do Iorb=1,Morb
         ORBTMP=(FTPSI(:,IORB))

         CALL get_FFT2D(-1,NDX,NDY,ORBTMP,INFO)
         call normvxz(ORBTMP,nrm,NDX*NDY)
         call xvixdzo(nrm,ORBTMP,NDX*NDY)
         
         call sort_FFT_to_ascending_2d(ORBTMP,NDX,NDY)

         FTPSI(:,IORB)=ORBTMP
     end do
  !##########################################################  
  !##########################################################  
  !     2D -CASE
  endif
  !##########################################################  
  !##########################################################

  
!---}}}
end subroutine FT_SORTED

!>-------------------------------------------------------------------------------
!> sort an array representing a 2D object given in FFT order, compact storage 
!> to be in strictly ascending order (needed for gnuplots pm3d), compact storage
!> \authors Axel Lode  (2011)
!> \since 2011
!> \todo Probably move this routine to the FFT module
subroutine sort_FFT_to_ascending_2d(FFT,NDX,NDY)
IMPLICIT NONE
integer NDX,NDY,i,j,k
COMPLEX*16 FFT(NDX*NDY),ASC(NDX*NDY)

         ASC=FFT
 
! copy the first half of each vector in X direction to the corresponding last half

   IF (MOD(NDX,2).eq.0) THEN
         do k=1,NDY
            do i=1,NDX
             if (i.le.((NDX)/2)) then
                FFT((k-1)*NDX+i)=ASC((k-1)*NDX+i+(NDX)/2)
             else if (i.gt.((NDX)/2)) then
                FFT((k-1)*NDX+i)=ASC((k-1)*NDX+i-((NDX)/2))
             endif
            end do
         end do 
   ENDIF


! copy the first half of each vector in X direction to the corresponding last half
   IF (MOD(NDX,2).ne.0) then
         do k=1,NDY
            do i=1,NDX
             if (i.le.((NDX+1)/2-1)) then
                FFT((k-1)*NDX+i)=ASC((k-1)*NDX+i+(NDX+1)/2)
             else if (i.gt.((NDX+1)/2)-1) then
                FFT((k-1)*NDX+i)=ASC((k-1)*NDX+i-((NDX+1)/2-1))
             endif
            end do
         end do 
   ENDIF

   IF (MOD(NDY,2).eq.0) then
! copy y direction from 0 -> kmax to last half

         do k=1,NDX*NDY
            if (k.le.(NDX*(NDY/2))) then
               ASC(k)=FFT(NDX*(NDY/2)+k)
            elseif (k.gt.NDX*((NDY)/2)) then
               ASC(k)=FFT(k-NDX*(NDY/2))
            endif
         end do
   ENDIF
   IF (MOD(NDY,2).ne.0) then
! copy y direction from 0 -> kmax to last half
         do k=1,NDX*NDY
            if (k.le.(NDX*(NDY/2))) then
               ASC(k)=FFT(NDX*(NDY/2+1)+k)
            elseif (k.gt.NDX*(NDY/2)) then
               ASC(k)=FFT(k-NDX*((NDY/2)))
            endif
         end do
   ENDIF
   FFT=ASC
end subroutine sort_FFT_to_ascending_2d

!>-------------------------------------------------------------------------------
!> This subroutine is called to generate the names of the output files of the
!> slices through the 2D correlation functions.
!> \authors Axel Lode  (2011)
!> \since 2011
!> \todo Probably move this routine to the Input/Output module
subroutine getsliceAsString(d,x,doubleString)
!       {{{
!          
IMPLICIT NONE
integer       :: d
integer,parameter         :: DBL=8
real(kind=DBL)            :: x
CHARACTER(len=7)         :: doubleString
CHARACTER(len=2)          :: slc

if (d.eq.1) then 
    write(slc,'(A2)') 'c1'
elseif (d.eq.2) then 
    write(slc,'(A2)') 'c2'
elseif (d.eq.3)  then 
    write(slc,'(A2)') 'c3'
elseif (d.eq.4) then 
    write(slc,'(A2)') 'c4'
else 
  write(*,*) 'This slice cannot be right!'
  stop
endif

if (x .LE. 9999.99999d0) WRITE(doubleString,'(F5.0,A2)') x,slc
if (x .LE. 999.999999d0) WRITE(doubleString,'(F5.1,A2)') x,slc
if (x .LE. 99.9999999d0) WRITE(doubleString,'(F5.2,A2)') x,slc
if (x .LE. 9.99999999d0) WRITE(doubleString,'(F5.3,A2)') x,slc

IF ((x .LT. 0.d0) .OR. (x .GE. 10000.0d0)) THEN
  WRITE(6,*)"get slice as string: x not implemented!"
  WRITE(doubleString,'(A7)') 'XXXXXXX' 
END IF

!}}}
end subroutine getsliceasString

END MODULE modulecorrelationfunctions
