! vim:fdm=marker:

!-------------------------------------------------------------------------------------------
!> Contains variable declarations and subroutines to write external potentials
!> with the additional program create_V_ext. Add whatever external
!> potential you like.
!>
!> \authors Kaspar Sakmann (2011)
!> \since 2011
MODULE modulecreate_V_ext
USE modulegrid
IMPLICIT NONE


CONTAINS


!-------------------------------------------------------------------------------------------
!>This routine writes an external potential V_ext to the file 'filename'
!> 
!> \authors Kaspar Sakmann (2011)
!> \since 2008
SUBROUTINE write_potentialToFile(NDX,NDY,NDZ,gridx,gridy,gridz,V_ext,filename)
! {{{ 
IMPLICIT NONE
INTEGER,INTENT(IN)        :: NDX,NDY,NDZ
REAL*8,INTENT(IN)         :: V_ext(NDX*NDY*NDZ)
REAL*8,INTENT(IN)         :: gridx(NDX),gridy(NDY),gridz(NDZ)
CHARACTER(150),INTENT(IN) :: filename
INTEGER                   :: outputunit,ierr
INTEGER                   :: i,j,k
REAL*8                    :: x,y,z

outputunit      = 11
ierr            = 0
x               = 0.d0
y               = 0.d0
z               = 0.d0

OPEN(UNIT=outputunit,FILE=filename,&
     FORM='formatted',ACTION='WRITE',STATUS='REPLACE',IOSTAT=ierr) 

  openif: IF (ierr == 0) THEN 

    writeloop: DO k=1,NDZ  
      DO j=1,NDY  
        DO i=1,NDX ! format: x,y,z, V(x,y,z)   
          x=gridx(i) 
          y=gridy(j) 
          z=gridz(k) 
          WRITE(outputunit,*)x,y,z,V_ext(i+NDX*(j-1)+NDX*NDY*(k-1)) 

        END DO
      END DO
    END DO writeloop

  ELSE openif
    WRITE(*,1040) ierr
    1040 FORMAT (' ','Error opening file: IOSTAT = ', I6)
  END IF openif

CLOSE(outputunit)


!       }}}
END SUBROUTINE write_potentialToFile 


!-------------------------------------------------------------------------------------------
!> This routine computes the 1D external trapping potential (bosonic Josephson
!> Junction) that  was used in the work of  Sakmann et al. PRL. 103, 220601 (2009)
!>
!> \authors Kaspar Sakmann (2008)
!> \since 2008
SUBROUTINE get_bjjLetterPotential(NDX,NDY,NDZ,gridx,gridy,gridz,Dims,V_ext) 
!--- {{{
IMPLICIT NONE

INTEGER,INTENT(IN)      :: NDX,NDY,NDZ,Dims
REAL*8,INTENT(IN)       :: gridx(NDX),gridy(NDY),gridz(NDZ)
REAL*8,INTENT(OUT)      :: V_ext(NDX*NDY*NDZ)
REAL*8                  :: x,y,z
REAL*8                  :: x0,xs,a,s0,s2,s3 !for a spline

INTEGER               :: i,j,k,m,n

! this defines a cubic (natural) spline that connects at
! +-xs to harmonic potentials a(x+-x0)**2. Used in the PRL of Sakmann et al. (2009).
x0 = -2.0d0
xs = -0.5d0 
a  = 0.5d0

! for the spline
s0 = 1.d0/3.d0 * (3.d0*a*x0**2 - 2*a*x0*xs)
s2 = (-2.d0*a*x0+a*xs)/xs
s3 = 2.d0/3.d0 * a*x0/xs**2

DO m=1,NDX*NDY*NDZ

  CALL get_ijk_from_m(m,NDX,NDY,i,j,k)
  x=gridx(i)
  y=gridy(j)
  z=gridz(k)
  
  IF (x< xs) THEN
    V_ext(m)=a*(x-x0)**2
  ELSE IF ((x >= xs) .and. (x<=0.d0)) THEN
    V_ext(m)=s0 + s2 * x**2 + s3 * x**3
  ELSE IF ((x > 0.d0) .and. (x<=-xs)) THEN
    V_ext(m)=s0 + s2 * (-x)**2 + s3 * (-x)**3
  ELSE IF ((x > -xs)) THEN
    V_ext(m)=a*(x+x0)**2
  END IF   

END DO

!---}}}
END SUBROUTINE get_bjjLetterPotential

!----------------------------------------------------------------------------------------------
!>this subroutine computes a harmonic external potential 
!>
!>  V_ext(x,y,z) = 1/2 ( w_x**2 (x-x0)**2 + w_y**2 (y-y0)**2 + w_z**2 (z-z0)**2 )
!>
!> \authors Kaspar Sakmann (2007)
!> \since 2007
SUBROUTINE get_harmonicPotential(NDX,NDY,NDZ,gridx,gridy,gridz,x0,y0,z0,w_x,w_y,w_z,V_ext) 
!--- {{{
IMPLICIT NONE

INTEGER,INTENT(IN)      :: NDX,NDY,NDZ
REAL*8,INTENT(IN)       :: gridx(NDX),gridy(NDY),gridz(NDZ),x0,y0,z0,w_x,w_y,w_z
REAL*8,INTENT(OUT)      :: V_ext(NDX*NDY*NDZ)
REAL*8                  :: x,y,z

INTEGER               :: i,j,k,m

DO m=1,NDX*NDY*NDZ

  CALL get_ijk_from_m(m,NDX,NDY,i,j,k)
  x=gridx(i)
  y=gridy(j)
  z=gridz(k)
  
  V_ext(m) = 0.5d0 *( w_x**2 * (x-x0)**2 + w_y**2 * (y-y0)**2 + w_z**2 * (z-z0)**2 )  

END DO

!---}}}
END SUBROUTINE get_harmonicPotential




END MODULE modulecreate_V_ext
