! vim:fdm=marker:
!------------------------------------------------------------------------------
!> This module contains routines that derive system parameters from
!> the variables read in via input.nml. 
!> The module ses the function beta of the asa189 library to compute
!> the number of configurations. This is useful for more than M=2 orbitals.
!>
!>\authors Kaspar Sakmann (2010)
!>\since (2010)
MODULE modulederivesystemvariables
IMPLICIT NONE
REAL*8,SAVE      :: weight



CONTAINS

!------------------------------------------------------------------------------
!> This routine gets the integration weight given the position grid.
!> Since we are using an equidistant grid there is only one 
!> integration weight. All orbitals are stored in the following form: the value !> of the orbital k at the grid point x_m,y_n,z_l is saved at the position
!> Psi(m+NDX*n+NDX*NDY*l,k) which contains the value of the orbital times the
!> weight, such that the norm of the orbital k
!> is one: norm^2 = Sum_j Psi(j,k)^* Psi(j,k). Thus, 
!> the weight is just the square root of the distance (1D) area (2D) volume (3D)!> between grid points.
!>
!> \authors Kaspar Sakmann (2007)
!> \since (2007)
SUBROUTINE get_weight(xi,xf,yi,yf,zi,zf,NDX,NDY,NDZ,Dims,weight)
!---{{{
IMPLICIT NONE
REAL*8,INTENT(IN)       :: xi,xf,yi,yf,zi,zf
INTEGER,INTENT(IN)      :: NDX,NDY,NDZ,Dims

REAL*8,INTENT(OUT)      :: weight
REAL*8                  :: dx,dy,dz,TOL

TOL     =       1.0D-6
weight  =       0.0d0
dx      =       0.0d0
dy      =       0.0d0
dz      =       0.0d0

IF (Dims == 1) THEN 

  dx = (xf-xi)/NDX
  IF (dx > TOL ) THEN 
    weight = DSQRT(dx)
  ELSE 
    WRITE(*,*)"dx = ",dx
    WRITE(*,*)"THERE IS PROBABLY SOMETHING WRONG WITH YOUR GRID. TO CONTINUE ANYWAY HIT ENTER"
    READ(*,*)
  END IF

ELSE IF (Dims == 2) THEN 

  dx = (xf-xi)/NDX
  dy = (yf-yi)/NDY
  IF (dx > TOL .AND. dy > TOL ) THEN 
    weight = DSQRT(dx*dy)
  ELSE 
    WRITE(*,*)"dx = ",dx
    WRITE(*,*)"dy = ",dy
    WRITE(*,*)"THERE IS PROBABLY SOMETHING WRONG WITH YOUR GRID. TO CONTINUE ANYWAY HIT ENTER"
    READ(*,*)
  END IF

ELSE IF (Dims == 3) THEN 

  dx = (xf-xi)/NDX
  dy = (yf-yi)/NDY
  dz = (zf-zi)/NDZ
  IF (dx > TOL  .AND. dy > TOL .AND. dz > TOL) THEN
    weight = DSQRT(dx*dy*dz)
  ELSE 
    WRITE(*,*)"dx = ",dx
    WRITE(*,*)"dy = ",dy
    WRITE(*,*)"dz = ",dz
    WRITE(*,*)"THERE IS PROBABLY SOMETHING WRONG WITH YOUR GRID. TO CONTINUE ANYWAY HIT ENTER"
    READ(*,*)
  END IF

ELSE 

  WRITE(*,*)"INVALID NUMBER OF DIMENSIONS Dims = ",Dims
  STOP

END IF

IF (DABS(weight) < 1.0D-10 ) THEN

  WRITE(*,*)"THERE IS PROBABLY SOMETHING WRONG WITH YOUR GRID, weight",weight

END IF

!---}}}
END SUBROUTINE get_weight

!-------------------------------------------------------------------------------------------
!> This routine computes the total number of configurations (permanents) given
!> the current system parameters NPar and MOrb: binomial(N+M-1,N). 
!> The routine uses the binom function of the asa189 library to compute
!> this number. The real value returned by binom is sufficiently 
!> accurate as long as the number nConf is less than about approx 10-50 x 10^6
!> see the file max_MOrb_NPar_parametersd0.1.txt for the largest values of M and N for which 
!> the real*8 result  differs from an int by less than d=0.1.
!>
!> \authors Kaspar Sakmann (2010)
!> \since (2010)
SUBROUTINE get_nConf(NPar,MOrb,nConf)
!---{{{ IMPLICIT NONE
INTEGER, INTENT(IN)      :: NPar,MOrb
INTEGER, INTENT(OUT)     :: nConf
real*8                   :: realval

realval=binomial(NPar+MOrb-1,NPar)
nConf=NINT(realval)


IF (DABS(realval-nConf)>0.05d0) THEN

  write(*,*)""
  write(*,*)"WATCH OUT, THE BETA FUNCTION BECOMES IMPRECISE!"
  write(*,*)"real      nConf ", realval
  write(*,*)"and  NINT(nConf)", nConf
  write(*,*)""
  read(*,*)

END IF

!---}}}
END  SUBROUTINE get_nConf

!-----------------------------------------------------------------------------
!> This routine computes the number of super diagonals of the Hamiltonian
!> matrix <n|H|n'> depending on the input variable MOrb. Only M=1 and M=2 
!> orbitals are supported. The Hamiltonian matrix can then be build and
!> given to the  LAPACK subroutine ZHBMV. For more orbitals it is 
!> not a good idea to keep  working with band matrix routines, because of the 
!> large number of zero elements in them.
!>
!>\authors Kaspar Sakmann (2009)
!> \since (2009)
SUBROUTINE get_nSuperDiags(NPar,MOrb,nSuperDiags)
!---{{{
IMPLICIT NONE

INTEGER, INTENT(IN)     :: NPar,MOrb
INTEGER, INTENT(OUT)     :: nSuperDiags


IF(MOrb .LT. 1) THEN
        WRITE(*,*)"INVALID VALUE FOR M",MOrb
        STOP
ENDIF
         
IF(MOrb == 1) nSuperDiags = 0
IF(MOrb == 2) nSuperDiags = 2
IF(MOrb > 2) THEN
        WRITE(*,*)"SORRY, M<=2 IS NECESSARY FOR USING ZHBMV EFFICIENTLY"
        STOP
ENDIF

IF(NPar==1) THEN

        IF(MOrb==1) THEN 
                nSuperDiags = 0
        ELSE 
                WRITE(*,*)"PLEASE SET M=1 FOR NPAR=1"
                STOP
        ENDIF

ENDIF
!---}}}
END  SUBROUTINE get_nSuperDiags

!------------------------------------------------------------------------------
!> This routine computes the total number of grid points and checks for consistency
!> 
!> \authors Kaspar Sakmann (2007)
!> \since (2007)
SUBROUTINE get_totalNoGridPts(NDX,NDY,NDZ,Dims,totalNoGridPts)
!---{{{
IMPLICIT NONE
INTEGER, INTENT(IN)      :: NDX,NDY,NDZ,Dims
INTEGER, INTENT(OUT)     :: totalNoGridPts

totalNoGridPts=NDX*NDY*NDZ

IF (totalNoGridPts <= 0) THEN 
  WRITE(*,*)"you messed up, total number of grid points:",totalNoGridPts
  STOP
END IF

IF (Dims==1 .AND. totalNoGridPts /= NDX) THEN
  WRITE(*,*) "INCONSISTENT GRID AND DIMENSION"
  STOP
END IF

IF (Dims==2 .AND. totalNoGridPts /= NDX*NDY) THEN
  WRITE(*,*) "INCONSISTENT GRID AND DIMENSION"
  STOP
END IF

IF (Dims==3 .AND. totalNoGridPts /= NDX*NDY*NDZ) THEN
  WRITE(*,*) "INCONSISTENT GRID AND DIMENSION"
  STOP
END IF


!---}}}
END SUBROUTINE get_totalNoGridPts

!------------------------------------------------------------------------------
!> This function computes the binomial coefficent (N over k) using the beta function.
!> the result is real and the integer is obtained by using NINT().
!> This way of computing binomial coefficients is very fast, but 
!> becomes inaccurate for results greater than dozens of millions.
!> Please see the routine get_nConf and the file max_MOrb_NPar_parametersd0.1 
!> for details
!>
!>\authors Kaspar Sakmann (2010)
!> \since (2010)
REAL*8 FUNCTION binomial(n,k)
!---{{{ 
IMPLICIT NONE
real*8                  :: beta
integer,intent(in)      :: N,K

binomial= 1.d0/beta(DBLE(N-K+1),DBLE(K+1)) * 1.d0/(N+1)
!---}}}
end function binomial




END MODULE modulederivesystemvariables
