! vim:fdm=marker:

!---------------------------------------------------------------------------------------
!> This module contains computational extensions, e.g. time-dependent extenal
!> potentials, time-dependent interaction strengths, etc.
!> 
!> \authors Kaspar Sakmann (2007)
!> \since 2007
MODULE moduleextensions
IMPLICIT NONE

CONTAINS


!-------------------------------------------------------------------------------------
!  (POSSIBLY) TIME DEPENDENT EXTERNAL POTENTIAL  
!> This subroutine is used if the potential is not read in (readPotential = F in
!> the input.nml file). the external potential defined here can be time-dependent.
!> put whatever you feel like.  
!> 
!> \authors Kaspar Sakmann (2007)
!> \since 2007
SUBROUTINE get_V_ext(V_ext,NDX,NDY,NDZ,Dims,gridx,gridy,gridz,AbsTime)
USE modulegrid
!--{{{
IMPLICIT NONE
INTEGER,INTENT(IN)      :: NDX,NDY,NDZ,Dims
REAL*8,INTENT(IN)       :: gridx(NDX),gridy(NDY),gridz(NDZ),AbsTime
REAL*8,INTENT(OUT)      :: V_ext(NDX*NDY*NDZ)

REAL*8                  :: x,y,z
INTEGER                 :: i,j,k,m

!e.g. a harmonic potential
REAL*8                  :: w_x=1.d0,w_y=1.d0,w_z=1.d0
REAL*8                  :: x0=0.d0,y0=0.d0,z0=0.d0

w_x             = 1.0d0  ! V(x)       = 0.5* w_x**2 x**2 
w_y             = 1.d00  ! V(y)       = 0.5* w_y**2 y**2 
w_z             = 1.0d0  ! V(z)       = 0.5* w_z**2 z**2 

DO m=1,NDX*NDY*NDZ

  CALL get_ijk_from_m(m,NDX,NDY,i,j,k)
  x=gridx(i)
  y=gridy(j)
  z=gridz(k)


  V_ext(m)= 0.5d0*w_x**2 * x**2 + 0.5d0*w_y**2 * y**2 +  0.5d0*w_z**2 * z**2

END DO


!---}}}
END SUBROUTINE get_V_ext



!----------------------------------------------------------------------------------
!> This function should be used if the interaction potential depends only on the 
!> distance between two particles and is not a delta funciton. 
!> In this case the IMEST algorithm  [K. Sakmann, PhD thesis, (2010)] 
!> can be used to evaluate all matrix elements Wsl efficiently. 
!> It is only used once to compute vtilde.
!>
!> Set useIMEST = T in input.nml to use this potential
!> 
!> Currently a unnormalized gaussian interaction is implemented 
!> here that converges towards
!> the delta function result if the width of the gaussian tends to zero.
!> Make sure you use an even number of grid points, because only then 
!> there will be a grid point at zero in the computation of vtilde.
!> If available, it is better to provide vtilde analytically.
!> Otherwise you will need a very dense grid.
!> So be careful what you put here as an interaction. The grid has to be
!> dense enough to resolve the interaction potential.
!>
!> \authors Kaspar Sakmann (2007)
!> \since 2007
!> \todo This is not fully implemented yet, input file option,
!> pass lambda0
REAL*8 FUNCTION Wint_IMEST(x,y,z)
USE moduleparameters
USE moduleinputvariables,ONLY:lambda0,Dims,imestParameter1
USE modulederivesystemvariables, ONLY: weight
IMPLICIT NONE
REAL*8,INTENT(IN)        :: x,y,z
REAL*8                   :: sigma

sigma           =      imestParameter1

Wint_IMEST = lambda0 / weight**2  * DEXP(-1.d0*(x**2+y**2+z**2)/(2.d0*sigma**2))

END FUNCTION Wint_IMEST

END MODULE moduleextensions
